# -*- coding: utf-8 -*-
#
# File: MeetingUser.py
#
# Copyright (c) 2010 by PloneGov
# Generator: ArchGenXML Version 1.5.2
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Gaetan DELANNAY <gaetan.delannay@geezteem.com>, Gauthier BASTIEN
<gbastien@commune.sambreville.be>, Stephan GEULETTE
<stephan.geulette@uvcw.be>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.PloneMeeting.config import *

##code-section module-header #fill in your manual code here
from Globals import InitializeClass
from Products.PloneMeeting.utils import getCustomAdapter, HubSessionsMarshaller

# Marshaller -------------------------------------------------------------------
class MeetingUserMarshaller(HubSessionsMarshaller):
    '''Allows to marshall a meeting user into a XML file.'''
    security = ClassSecurityInfo()
    security.declareObjectPrivate()
    security.setDefaultAccess('deny')
    fieldsToMarshall = 'all'
    rootElementName = 'meetingUser'
InitializeClass(MeetingUserMarshaller)
##/code-section module-header

schema = Schema((

    StringField(
        name='title',
        widget=StringWidget(
            visible=False,
            label='Title',
            label_msgid='PloneMeeting_label_title',
            i18n_domain='PloneMeeting',
        ),
        accessor="Title"
    ),

    StringField(
        name='configId',
        index="FieldIndex",
        widget=StringWidget(
            visible=False,
            label='Configid',
            label_msgid='PloneMeeting_label_configId',
            i18n_domain='PloneMeeting',
        )
    ),

    StringField(
        name='duty',
        widget=StringWidget(
            description="MeetingUserDuty",
            description_msgid="meeting_user_duty_descr",
            condition="python: here.isManager()",
            label='Duty',
            label_msgid='PloneMeeting_label_duty',
            i18n_domain='PloneMeeting',
        )
    ),

    LinesField(
        name='usages',
        widget=MultiSelectionWidget(
            description="MeetingUserUsages",
            description_msgid="meeting_user_usages_descr",
            format="checkbox",
            condition="python: here.isManager()",
            label='Usages',
            label_msgid='PloneMeeting_label_usages',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        multiValued=1,
        vocabulary='listUsages'
    ),

    ImageField(
        name='signatureImage',
        widget=ImageWidget(
            description="MeetingUserSignatureImage",
            description_msgid="meeting_user_signature_image_descr",
            condition="python: here.isManager()",
            label='Signatureimage',
            label_msgid='PloneMeeting_label_signatureImage',
            i18n_domain='PloneMeeting',
        ),
        storage=AttributeStorage()
    ),

    BooleanField(
        name='signatureIsDefault',
        default= False,
        widget=BooleanField._properties['widget'](
            description="MeetingUserSignatureIsDefault",
            description_msgid="meeting_user_signature_is_default",
            condition="python: here.isManager()",
            label='Signatureisdefault',
            label_msgid='PloneMeeting_label_signatureIsDefault',
            i18n_domain='PloneMeeting',
        )
    ),

    StringField(
        name='meetingAppDefaultView',
        widget=SelectionWidget(
            description="MeetingAppDefaultView",
            description_msgid="meeting_app_default_view_descr",
            label='Meetingappdefaultview',
            label_msgid='PloneMeeting_label_meetingAppDefaultView',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        vocabulary='listMeetingAppAvailableViews',
        default_method='getMeetingAppDefaultValue'
    ),

    StringField(
        name='mailFormat',
        widget=SelectionWidget(
            description="MailFormat",
            description_msgid="mail_format_descr",
            label='Mailformat',
            label_msgid='PloneMeeting_label_mailFormat',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        vocabulary='listMailFormats',
        default_method='getDefaultMailFormat'
    ),

    StringField(
        name='adviceStyle',
        widget=SelectionWidget(
            description="AdviceStyle",
            description_msgid="advice_style_descr",
            label='Advicestyle',
            label_msgid='PloneMeeting_label_adviceStyle',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary= True,
        vocabulary='listAdviceStyles',
        default_method="getDefaultAdviceStyle"
    ),

    LinesField(
        name='itemsListVisibleColumns',
        widget=MultiSelectionWidget(
            description="ItemsListVisibleColumns",
            description_msgid="items_list_visible_columns_descr",
            format="checkbox",
            label='Itemslistvisiblecolumns',
            label_msgid='PloneMeeting_label_itemsListVisibleColumns',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        multiValued=1,
        vocabulary='listItemsListVisibleColumns',
        default_method="getDefaultVisibleColumns"
    ),

    LinesField(
        name='itemColumns',
        widget=MultiSelectionWidget(
            description="ItemColumns",
            description_msgid="item_columns_descr",
            format="checkbox",
            label='Itemcolumns',
            label_msgid='PloneMeeting_label_itemColumns',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        multiValued=1,
        vocabulary="listItemColumns",
        default_method="getDefaultItemColumns"
    ),

    LinesField(
        name='meetingColumns',
        widget=MultiSelectionWidget(
            description="MeetingColumns",
            description_msgid="meeting_columns_descr",
            format="checkbox",
            label='Meetingcolumns',
            label_msgid='PloneMeeting_label_meetingColumns',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        multiValued=1,
        vocabulary="listMeetingColumns",
        default_method="getDefaultMeetingColumns"
    ),

    BooleanField(
        name='openAnnexesInSeparateWindows',
        widget=BooleanField._properties['widget'](
            description="OpenAnnexesInSeparateWindows",
            description_msgid="open_annexes_in_separate_windows_descr",
            label='Openannexesinseparatewindows',
            label_msgid='PloneMeeting_label_openAnnexesInSeparateWindows',
            i18n_domain='PloneMeeting',
        ),
        default_method="getOpenAnnexesDefaultValue"
    ),

    LinesField(
        name='mailItemEvents',
        widget=MultiSelectionWidget(
            description="MailItemEvents",
            description_msgid="mail_item_events_descr",
            condition="python: here.listItemEvents()",
            format="checkbox",
            label='Mailitemevents',
            label_msgid='PloneMeeting_label_mailItemEvents',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        multiValued=1,
        vocabulary="listItemEvents",
        default_method="getDefaultItemEvents"
    ),

    LinesField(
        name='mailMeetingEvents',
        widget=MultiSelectionWidget(
            description="MailMeetingEvents",
            description_msgid="mail_meeting_events",
            condition="python: here.listMeetingEvents()",
            format="checkbox",
            label='Mailmeetingevents',
            label_msgid='PloneMeeting_label_mailMeetingEvents',
            i18n_domain='PloneMeeting',
        ),
        enforceVocabulary=True,
        multiValued=1,
        vocabulary="listMeetingEvents",
        default_method="getDefaultMeetingEvents"
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

MeetingUser_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
from Products.PloneMeeting.model.extender import ModelExtender
MeetingUser_schema = ModelExtender(MeetingUser_schema, 'muser').run()
# Register the marshaller for DAV/XML export.
MeetingUser_schema.registerLayer('marshall', MeetingUserMarshaller())
##/code-section after-schema

class MeetingUser(BaseContent):
    """
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseContent,'__implements__',()),)

    # This name appears in the 'add' box
    archetype_name = 'MeetingUser'

    meta_type = 'MeetingUser'
    portal_type = 'MeetingUser'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 1
    #content_icon = 'MeetingUser.gif'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "MeetingUser"
    typeDescMsgId = 'description_edit_meetinguser'


    actions =  (


       {'action': "string:$object_url/base_metadata",
        'category': "object",
        'id': 'metadata',
        'name': 'Properties',
        'permissions': ("Manage portal",),
        'condition': 'python:1'
       },


    )

    _at_rename_after_creation = True

    schema = MeetingUser_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    # Manually created methods

    security.declarePublic('getSelf')
    def getSelf(self):
        if self.__class__.__name__ != 'MeetingUser': return self.context
        return self

    security.declarePublic('adapted')
    def adapted(self): return getCustomAdapter(self)

    security.declareProtected('Modify portal content', 'onEdit')
    def onEdit(self, isCreated): '''See doc in interfaces.py.'''

    security.declarePublic('config')
    def config(self):
        return self.getParentNode().getParentNode()

    security.declarePrivate('updateMeetingUser')
    def updateMeetingUser(self):
        '''Updates this meeting user (local roles, title).'''
        # Updates the title
        userInfo = self.portal_membership.getMemberById(self.id)
        if userInfo and userInfo.getProperty('fullname'):
            title = userInfo.getProperty('fullname')
        else:
            title = self.id
        self.setTitle(title)
        self.setConfigId(self.config().id)
        # Update local roles. Corresponding Plone user own me.
        if 'Owner' not in self.get_local_roles_for_userid(self.id):
            self.manage_addLocalRoles(self.id, ('Owner',))

    security.declarePrivate('at_post_create_script')
    def at_post_create_script(self):
        self.updateMeetingUser()
        self.adapted().onEdit(isCreated=True)

    security.declarePrivate('at_post_edit_script')
    def at_post_edit_script(self):
        self.updateMeetingUser()
        self.adapted().onEdit(isCreated=False)

    def listUsages(self):
        '''Returns list of possible usages (for what will this user be useful
           in voting process: "assembly member", "signer" or "voter").'''
        d = 'PloneMeeting'
        res = DisplayList((
                ("assemblyMember", self.utranslate(
                    'meeting_user_usage_assemblyMember', domain=d)),
                ("signer", self.utranslate(
                    'meeting_user_usage_signer', domain=d)),
                ("voter", self.utranslate(
                    "meeting_user_usage_voter", domain=d)),
              ))
        return res

    security.declarePublic('mayConsultVote')
    def mayConsultVote(self, loggedUser, item):
        '''See doc in interfaces.py.'''
        mUser = self.getSelf()
        if (loggedUser.id == mUser.getId()) or \
           loggedUser.has_role('MeetingManager') or \
           loggedUser.has_role('Manager') or \
           item.getMeeting().isDecided():
            return True
        return False

    security.declarePublic('mayEditVote')
    def mayEditVote(self, loggedUser, item):
        '''See doc in interfaces.py.'''
        mUser = self.getSelf()
        if loggedUser.has_role('Manager'):
            return True
        if item.getMeeting().isDecided():
            return False
        else:
            meetingConfig = item.portal_plonemeeting.getMeetingConfig(item)
            votesEncoder = meetingConfig.getVotesEncoder()
            if (loggedUser.id == mUser.getId()) and \
               ('theVoterHimself' in votesEncoder):
                return True
            if loggedUser.has_role('MeetingManager') and \
               ('aMeetingManager' in votesEncoder):
                return True
        return False

    security.declarePublic('isManager')
    def isManager(self):
        '''Has logged user role Manager?'''
        user = self.portal_membership.getAuthenticatedMember()
        return user.has_role('Manager')

    def listMeetingAppAvailableViews(self):
        return self.config().listMeetingAppAvailableViews()

    def getMeetingAppDefaultValue(self):
        return self.config().getMeetingAppDefaultView()

    def listMailFormats(self):
        return self.config().listMailFormats()

    def getDefaultMailFormat(self):
        return self.config().getMailFormat()

    def listAdviceStyles(self):
        return self.config().listAdviceStyles()

    def getDefaultAdviceStyle(self):
        return self.config().getAdviceStyle()

    def listItemsListVisibleColumns(self):
        return self.config().listItemsListVisibleColumns()

    def getDefaultVisibleColumns(self):
        return self.config().getItemsListVisibleColumns()

    def listItemColumns(self):
        return self.config().listItemColumns()

    def getDefaultItemColumns(self):
        return self.config().getItemColumns()

    def listMeetingColumns(self):
        return self.config().listMeetingColumns()

    def getDefaultMeetingColumns(self):
        return self.config().getMeetingColumns()

    def getOpenAnnexesDefaultValue(self):
        return self.config().getOpenAnnexesInSeparateWindows()

    def listItemEvents(self):
        config = self.config()
        allEvents = config.listItemEvents()
        selectedEvents = config.getMailItemEvents()
        # From the list of all events that are selectable in the config
        # (=configItemEvents), the user may only (de)activate events
        # that are selected in the config (=selectedEvents). Among
        # selectedEvents, the user may only (de)activate events that make sense
        # (ie, if the user is not an adviser, it has no sense to propose him to
        # (de)activate events sent to an adviser).
        res = []
        # We must know if the user is a MeetingManager, an item creator or
        # adviser
        tool = config.getParentNode()
        user = tool.portal_membership.getAuthenticatedMember()
        isMeetingManager = user.has_role('MeetingManager')
        isCreator = tool.userIsAmong('creators')
        isAdviser = tool.userIsAmong('advisers')
        for event in selectedEvents:
            keepIt = False
            if (event in ('lateItem', 'annexAdded', 'askDiscussItem')) and \
               isMeetingManager:
                keepIt = True
            elif event in ('itemPresented', 'itemUnpresented', 'itemDelayed') \
                and isCreator:
                keepIt = True
            elif event in ('adviceInvalidated',) and isAdviser:
                keepIt = True
            elif event == 'adviceEdited':
                keepIt = True # This event requires permission "View".
            if keepIt:
                res.append( (event, allEvents.getValue(event)) )
        return DisplayList(tuple(res))

    def getDefaultItemEvents(self):
        return self.listItemEvents().keys()

    def listMeetingEvents(self):
        config = self.config()
        allEvents = config.listMeetingEvents()
        # On an archive site, there may be no transition defined on a meeting.
        if not allEvents.keys(): return DisplayList()
        selectedEvents = config.getMailMeetingEvents()
        res = []
        # Here, I could only keep transitions for which the user has the
        # permission to View the meeting at its end state. But in most cases,
        # meeting-related events can be seen by everyone.
        for event in selectedEvents:
            res.append( (event, allEvents.getValue(event)) )
        return DisplayList(tuple(res))

    def getDefaultMeetingEvents(self):
        return self.listMeetingEvents().keys()

    security.declarePublic('indexUsages')
    def indexUsages(self):
        '''Returns the index content for usages.'''
        return self.getUsages()



registerType(MeetingUser, PROJECTNAME)
# end of class MeetingUser

##code-section module-footer #fill in your manual code here
##/code-section module-footer



