# ------------------------------------------------------------------------------
from persistent.list import PersistentList
from Products.PloneMeeting.profiles.migrations import Migrator
from Products.PloneMeeting.config import *
import logging
logger = logging.getLogger('PloneMeeting')

# The migration class ----------------------------------------------------------
class Migrate_To_1_8(Migrator):
    def __init__(self, context):
        Migrator.__init__(self, context)

    def _addItemHistory(self):
        '''This method adds a persistent list on every item for storing item
           history.'''
        logger.info('Adds on every item the persistent list that stores item ' \
                    'history.')
        brains = self.portal.portal_catalog(meta_type='MeetingItem')
        for brain in brains:
            item = brain.getObject()
            modifDate = item.modification_date
            # Creates the persistent list
            if not hasattr(item.aq_base, 'itemHistory'):
                item.itemHistory = PersistentList()
            item.setModificationDate(modifDate)
            # We ensure that the modification date of items has not changed
            # due to these maintenance-related changes.

    def _addOcrAttributesAndUpdateSecurity(self):
        '''Adds attributes "needsOcr" and "ocrLanguage" to every MeetingFile
           instance. Also adapts annex security: in some cases, a workflow was
           selected for the MeetingFile type. Now that no workflow is defined
           (see AppInstall.py) we have to ensure that the annexes security is
           correct by updating it on every element.'''
        logger.info('Adds on every annex attributes "needsOcr" and ' \
                    '"ocrLanguage" and updates annex security...')
        brains = self.portal.portal_catalog(meta_type='MeetingFile')
        for brain in brains:
            annex = brain.getObject()
            modifDate = annex.modification_date
            # Adds the attributes
            annex.needsOcr = False
            annex.ocrLanguage = None
            # Updates security
            annex.updateAnnexSecurity()
            annex.setModificationDate(modifDate)
            # We ensure that the modification date of annexes has not changed
            # due to these maintenance-related changes.

    def run(self):
        self.reinstall()
        self._addItemHistory()
        self._addOcrAttributesAndUpdateSecurity()
        self.refreshDatabase(catalogs=False, workflows=True)
        logger.info('Migration finished.')

# The migration function -------------------------------------------------------
def migrate(context):
    '''This migration function does the following things:

       0) Reinstall PloneMeeting;
       1) Adds a persistent list on every item for storing item's history;
       2) Adds new OCR-related attributes on MeetingFile;
       3) Updates annex security on annexes.'''
    Migrate_To_1_8(context).run()
# ------------------------------------------------------------------------------
